#include "UserDb.h"
//#define DEBUG
using namespace userdb;

UserDb::UserDb(const char *filename) : filename(filename)
{
}
void UserDb::coursor_to_end_of_line(File &f)
{
    while (f.available() && f.read() != '\n')
    {
    }
}
bool UserDb::remove_user(User &user)
{
    return update_db_line(filename, user.line);
}
bool UserDb::update_user(User *user)
{
    return update_db_line(filename, user->line, user);;
}
bool UserDb::update_db_line(const char *file, linenumber line, User *replace)
{
    const char *temp_file = ".tmp.csv";
    File old_db = LittleFS.open(file, "r");
    File new_db = LittleFS.open(temp_file, "w+");
    linenumber curr_line = 0;
    bool line_found = false;
    while (old_db.available())
    {
        if (curr_line == line)
        {

#ifdef DEBUG
            Serial.println("Found line: " + String(line));
#endif
            if (replace != nullptr)
            {
                String insert = replace->toCSVString();
#ifdef DEBUG
                Serial.println("Inserting: " + insert);
#endif
                if (new_db.position() > 0)
                    new_db.write('\n');
                new_db.write(insert.c_str());
            }
            coursor_to_end_of_line(old_db);
            line_found = true;
        }

        else
        {
            if (new_db.position() > 0)
                new_db.write('\n');
            old_db.sendUntil(new_db, '\n');
        }
        curr_line++;
    }
    old_db.close();
#ifdef DEBUG
    Serial.println("NewDB:");
    new_db.seek(0);
    while (new_db.available())
        new_db.sendAvailable(Serial);
#endif
    new_db.close();
    if (LittleFS.remove(file) && LittleFS.rename(temp_file, file))
        Serial.println("UserDb updated");
    else
        Serial.println("Could not delete old userdb");
    return line_found;
}
bool UserDb::add_user(User &user)
{

    File db = LittleFS.open(filename, "a");
    if (db)
    {
        if (db.size() > 0)
            db.write("\n");
        String csvline = user.toCSVString();
        db.write(csvline.c_str());
        db.close();
        return true;
    }
    else
        return false;
}

User UserDb::user_by_pin(String cmp)
{
#ifdef DEBUG
    Serial.println("Searching for user with pin number " + cmp);
#endif
    Iterator it = begin_with_filter(&cmp, USERATTRIBUTES::USER_PIN);
    do
    {
        User temp = *it;
        if ((*it).match && (*it).enabled)
        {
#ifdef DEBUG
            Serial.println(" -> Found match");
#endif
            it.close_file();
            return *it;
        }
        ++it;
    } while (it.has_next());
    return User{.enabled = false};
}
User UserDb::user_by_rfid(String cmp)
{
#ifdef DEBUG
    Serial.println("Searching for user with rfid " + cmp);
#endif
    Iterator it = begin_with_filter(&cmp, USERATTRIBUTES::RFID_UID);
    do
    {
        if ((*it).match && (*it).enabled)
        {
#ifdef DEBUG
            Serial.println(" -> Found match");
#endif
            it.close_file();
            return *it;
        }
        ++it;
    } while (it.has_next());
    return User{.enabled = false};
}

User UserDb::user_by_uid(unsigned long cmp)
{
    String cmps = String(cmp);
    return user_by_uid(cmps);
}
User UserDb::user_by_uid(String cmp)
{
#ifdef DEBUG
    Serial.println("Searching for user with id " + cmp);
#endif
    Iterator it = begin_with_filter(&cmp, USERATTRIBUTES::USER_ID);
    do
    {
        if ((*it).match)
        {
#ifdef DEBUG
            Serial.println(" -> Found match");
#endif
            it.close_file();
            return *it;
        }
        ++it;
    } while (it.has_next());
    return User{.enabled = false};
}
User UserDb::user_by_line(linenumber cmp)
{
#ifdef DEBUG
    Serial.println("Searching for user with line number " + String(cmp));
#endif
    Iterator it = begin_with_filter(cmp, USERATTRIBUTES::LINE_ID);
    do
    {
        if ((*it).match)
        {
#ifdef DEBUG
            Serial.println(" -> Found match");
#endif
            it.close_file();
            return (*it);
        }
        ++it;
    } while (it.has_next());
    return User{.enabled = false};
}
void UserDb::print_to_serial()
{
    Serial.println("Starting UserDB Test Read");
    for (Iterator it = begin(); it.has_next(); ++it)
    {
        User temp = *it;
        Serial.println(temp.toString());
    }
}
