#pragma once
#include <ESP8266mDNS.h>
#include <DNSServer.h>
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <uri/UriBraces.h>
#include <uri/UriRegex.h>
#include "LittleFS.h"
#include "ArduinoJson.h"
#include "UserDb.h"
#include "Rfid.h"
#include "Config.h"
#include "AdminAuth.h"
#include "Interface.h"
namespace webconsole
{
    static const char path_prefix[] PROGMEM = "/s";
    static ESP8266WebServer _server(80);
    class WebConsole
    {
    public:
        WebConsole();
        ~WebConsole();

        bool init(Config *config, userdb::UserDb *userdb);
        void attachRfid(Rfid *rfid);
        void serve();
        uint8_t isInterceptingRfid();

    private:
        void _sendCORS();
        void _settings();
        void _auth();
        bool _isAuth();
        void _handleUnknown();
        void _getUserDb();
        void _deleteUser();
        void _getUser();
        void _updateUser();
        void _createUser();
        void _dropUserDb();
        void _catchRFID();
        void _updateAdmin();
        void _print_db_raw()
        {
            File f = LittleFS.open("userdb.csv", "r");
            while (f.available())
            {
                f.sendAvailable(Serial);
            }
            _server->send(200, "text/plain", "printed to serial");
            f.close();
        }
        Config *_config;
        DNSServer *_dnsServer;
        ESP8266WebServer *_server;
        userdb::UserDb *userdb = nullptr;
        unsigned long catch_rfid_millis = 0;
        uint8_t catch_rfid = 0;
        bool catch_rfid_updated = false;
        String rfid_buffer;
        Rfid *rfid = nullptr;
        Interface *iface = nullptr;
        AdminAuth auth;
    };

}